unit frmSPCommDemo;

interface

uses
     XPMan,
     wxoExec,wxoUnit,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, SPComm, StdCtrls;

type
  TFormMain = class(TForm)
    pnlCommand: TPanel;
    btnOpen: TButton;
    btnClose: TButton;
    btnSend: TButton;
    grpReceived: TGroupBox;
    mmoReceived: TMemo;
    grpSend: TGroupBox;
    Splitter1: TSplitter;
    mmoSend: TMemo;
    grpError: TGroupBox;
    MmoError: TMemo;
    grpComSetting: TGroupBox;
    cmbCommName: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    cmbBaudrate: TComboBox;
    Label3: TLabel;
    cmbDatabits: TComboBox;
    Label4: TLabel;
    cmbStopbits: TComboBox;
    Label5: TLabel;
    cmbParity: TComboBox;
    Label6: TLabel;

    procedure FormCreate(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnSendClick(Sender: TObject);
    procedure mmoReceivedDblClick(Sender: TObject);
  private
    { Private declarations }
    FComm: TComm;

    procedure CreateCommPorts;
    function GetParity(AIndex: Integer): TParity;
    function GetStopBits(AIndex: Integer): TStopBits;
    function GetByteSize(AIndex: Integer): TByteSize;

    procedure ReceiveError(Sender: TObject; EventMask : DWORD);
    procedure ReceiveData(Sender: TObject; Buffer: Pointer; BufferLength: Word);
  public
    { Public declarations }
    procedure ToggleEnabled;
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

procedure TFormMain.FormCreate(Sender: TObject);
begin
     //
     wxoPrepare(Handle,Application.Handle);

  mmoSend.Clear;
  MmoError.Clear;
  mmoReceived.Clear;
  FComm := TComm.Create(nil);
  CreateCommPorts;

  ToggleEnabled;
end;

procedure TFormMain.CreateCommPorts;
var
  I: Integer;
begin
  with cmbCommName do
  begin
    Items.Clear;
    for I := 1 to 254 do
      Items.Add(Format('COM%d:', [I]));

    ItemIndex := 0;
  end;
end;

procedure TFormMain.btnOpenClick(Sender: TObject);
begin
  with FComm do
  begin
    try
      CommName := cmbCommName.Text;
      Parity := GetParity(cmbParity.ItemIndex);
      StopBits := GetStopBits(cmbStopbits.ItemIndex);
      ByteSize := GetByteSize(cmbDatabits.ItemIndex);
      BaudRate := StrToIntDef(cmbBaudrate.Text, 9600);
      ParityCheck := True;

      OnReceiveError := ReceiveError;
      OnReceiveData := ReceiveData;

      StopComm;
      StartComm;

      ToggleEnabled;
    except
      on E: Exception do
      begin
        mmoError.Lines.Add(E.Message)
      end;
    end;
  end;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FComm);
end;

function TFormMain.GetByteSize(AIndex: Integer): TByteSize;
begin
  Result := _8;
  case AIndex of
    0: Result := _5;
    1: Result := _6;
    2: Result := _7;
    3: Result := _8;
  end;
end;

function TFormMain.GetParity(AIndex: Integer): TParity;
begin
  Result := None;
  case AIndex of
    0: Result := None;
    1: Result := Odd;
    2: Result := Even;
  end;
end;

function TFormMain.GetStopBits(AIndex: Integer): TStopBits;
begin
  Result := _1;
  case AIndex of
    0: Result := _1;
    1: Result := _1_5;
    2: Result := _2;
  end;
end;

procedure TFormMain.mmoReceivedDblClick(Sender: TObject);
begin
  (Sender as TMemo).Clear;
end;

procedure TFormMain.btnCloseClick(Sender: TObject);
begin
  with FComm do
  begin
    StopComm;
  end;

  ToggleEnabled;
end;

procedure TFormMain.ReceiveData(Sender: TObject; Buffer: Pointer; BufferLength: Word);
var
  S: AnsiString;
begin
  SetLength(S, BufferLength);
  Move(Buffer^, PAnsiChar(S)^, BufferLength);

  mmoReceived.Lines.Add(S);
end;

// EventMaskȡֵ: μ http://bbs.csdn.net/topics/60311989
procedure TFormMain.ReceiveError(Sender: TObject; EventMask: DWORD);
var
  S: string;
begin
  with MmoError do
  begin
    Lines.Add(Format('OnReceiveError: EventMask=%d', [EventMask]));

    S := '';
    case EventMask of
      CE_BREAK:     // Ӳ⵽иն. (ڲ֧)
        S := 'Ӳ⵽иֹ.';
      CE_DNS:       // win95: ûѡӦ.
        S := 'win95: ûѡӦ.';
      CE_FRAME:     // ⵽и.
        S := '⵽и.';
      CE_IOE:       // 豸ͨгһI/O.
        S := '豸ͨгһI/O.';
      CE_MODE:      // Ҫģʽ֧, hFile ĲǷǷġ
        S := 'Ҫģʽ֧, hFile ĲǷǷġ';
      CE_OOP:       // win95: ӦļķΧ
        S := 'win95: ӦļķΧ';
      CE_OVERRUN:   // ַݶʧ
        S := 'ַݶʧ';
      CE_PTO:       // win95: Ӧ豸ʹʱ¼ʱ.
        S := 'win95: Ӧ豸ʹʱ¼ʱ.';
      CE_RXOVER:    // 뻺ַյļսʾֽӵַ
        S := '뻺ַյļսʾֽӵַ';
      CE_RXPARITY:  // ż
        S := 'ż';
      CE_TXFULL:    // £ַ
        S := '£ַ';
    end;

    if (S <> '') then Lines.Add('   ' + S);
  end;
end;

procedure TFormMain.ToggleEnabled;
var
  ACanSend: Boolean;
begin
  ACanSend := FComm.Connected;

  grpComSetting.Enabled := not ACanSend;
  btnOpen.Enabled := not ACanSend;
  btnClose.Enabled := ACanSend;
  btnSend.Enabled := ACanSend;
end;

procedure TFormMain.btnSendClick(Sender: TObject);
var
  S: AnsiString;
begin
  S := mmoSend.text;

  FComm.WriteCommData(@S[1], length(S));
end;

end.


